﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Design;

namespace RagPong
{
    class Bounceable
    {
        public Vector2 speed;
        public Texture2D avatar;
        public BoundingSphere shell;

        //default constructor: gives default speed, shell initialized
        public Bounceable()
        {
            setDefaultSpeed();
            shell = new BoundingSphere();

        }

        //returns the lowest y value of shell
        public float getUpperBound()
        {
            return shell.Center.Y - shell.Radius;
        }

        //returns the highest x value of the shell
        public float getRightBound()
        {
            return shell.Center.X + shell.Radius;
        }

        //returns the highest y value of the shell
        public float getLowerBound()
        {
            return shell.Center.Y + shell.Radius;
        }

        //returns the lowest x value of the shell
        public float getLeftBound()
        {
            return shell.Center.X - shell.Radius;
        }

        //shifts the postion of the shell around the ball by the amount 
        //specified by the vector passed in
        public void changePosition(Vector2 change)
        {
            shell.Center.X += change.X;
            shell.Center.Y += change.Y;
        }

        //returns whether or not the ball has collided with a paddle
        public virtual bool Collision(paddle p)
        {
            return shell.Intersects(p.box);
        }

        //this function draws the sprite to the screen
        public virtual void draw(SpriteBatch sB)
        {
            //no rotation, good collision
            sB.Draw(avatar, new Vector2(shell.Center.X - shell.Radius,
                shell.Center.Y - shell.Radius), Color.White);

        }

        public virtual void setDefaultSpeed()
        {
            speed = new Vector2(100.0f, 100.0f);
        }

    }
}