using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace RagPong
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        KeyboardState previousKeyboardState;
        Random rGen = new Random();
        int powerIndex;
        double timer = 0;
        double previousTimer = 0;
        float millisecondsUntilPowerup;
        float millisecondsLastPowerup;

        powerUp[] ups;
        ball pBall = new ball();
        player lPlayer = new player();
        player rPlayer = new player();
        string winner;
        bool gameOver = false;

        Menu mainMenu;
        Menu optionsMenu;
        bool inMainMenu = true;
        bool inCredits = false;
        bool inOptions = false;
        bool inControls = false;
        bool soundOn = true;
        bool musicOn = true;
        int scoreLimit = 3;

        Texture2D address_rags;
        Texture2D arab_rags;
        Texture2D greece_rags;
        Texture2D toga_rags;

        SoundEffect high;
        SoundEffect see;
        SoundEffect stud;
        SoundEffect scud;

        SoundEffect menuSelect;
        SoundEffect menuNavigate;
        Song backgroundMusic;

        string currentSoundState = "On";
        string currentMusicState = "On";
        string fullScreenState = "Off";

        int windowWidth;
        int windowHeight;
        bool fullScreen = false;
        Texture2D logo;

        Texture2D scrollingBackground;
        Texture2D scrollingBackground2;
        private float scrollY;
        private float scrollY2;
        private float backgroundHeight;

        SpriteFont Arial;
        SpriteFont Impact;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            windowHeight = graphics.GraphicsDevice.Viewport.Height;
            windowWidth = graphics.GraphicsDevice.Viewport.Width;

            graphics.PreferredBackBufferWidth = windowWidth;
            graphics.PreferredBackBufferHeight = windowHeight;
            graphics.IsFullScreen = fullScreen;
            graphics.ApplyChanges();
            Window.Title = "E X ^ R A G S 1.0";

            ups = new powerUp[3];
            for (int a = 0; a < 3; a++)
            {
                ups[a] = new powerUp();
                ups[a].augment = new paddle();
                ups[a].active = false;
            }

            ups[2].augment.speed *= 2;

            rPlayer.currentPU.augment = new paddle();
            lPlayer.currentPU.augment = new paddle();

            base.Initialize();
        }

        private void changeResolution()
        {
            graphics.PreferredBackBufferWidth = windowWidth;
            graphics.PreferredBackBufferHeight = windowHeight;
            graphics.IsFullScreen = fullScreen;
            graphics.ApplyChanges();

            //the screen size is reset
            setScreen();

        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            //spritefont: Arial, 14 point
            Arial = Content.Load<SpriteFont>("font");
            Impact = Content.Load<SpriteFont>("score");
            logo = Content.Load<Texture2D>("logo");

            //initializing main menu functionality
            mainMenu = new Menu(Color.White, Color.Red, Arial);
            mainMenu.addMenuItem("New Game", new Vector2(windowWidth * 0.2f, 100));
            mainMenu.addMenuItem("Controls", new Vector2(windowWidth * 0.2f, 150));
            mainMenu.addMenuItem("Options", new Vector2(windowWidth * 0.2f, 200));
            mainMenu.addMenuItem("Credits", new Vector2(windowWidth * 0.2f, 250));
            mainMenu.addMenuItem("Exit", new Vector2(windowWidth * 0.2f, 300));

            optionsMenu = new Menu(Color.White, Color.Red, Arial);
            optionsMenu.addMenuItem("Score:", new Vector2(50, 50));
            optionsMenu.addMenuItem("Sound:", new Vector2(50, 100));
            optionsMenu.addMenuItem("Music:", new Vector2(50, 150));
            optionsMenu.addMenuItem("Resolution:", new Vector2(50, 200));
            optionsMenu.addMenuItem("Fullscreen:", new Vector2(50, 250));
            optionsMenu.addMenuItem("Back", new Vector2(50, 300));

            //Scott Ragsdale sound effects
            high = Content.Load<SoundEffect>("Sounds\\rags\\high");
            stud = Content.Load<SoundEffect>("Sounds\\rags\\stud");
            scud = Content.Load<SoundEffect>("Sounds\\rags\\scud");
            see = Content.Load<SoundEffect>("Sounds\\rags\\see_who");

            //Scott Ragsdale menu images
            address_rags = Content.Load<Texture2D>("address");
            arab_rags = Content.Load<Texture2D>("arab");
            greece_rags = Content.Load<Texture2D>("greece");
            toga_rags = Content.Load<Texture2D>("toga");

            // scrolling background image
            scrollingBackground = Content.Load<Texture2D>("BST1");
            scrollingBackground2 = Content.Load<Texture2D>("BST2");
            backgroundHeight = scrollingBackground.Height;
            scrollY = 0;
            scrollY2 = backgroundHeight;

            //sound effects
            pBall.paddleHit = Content.Load<SoundEffect>("Sounds\\menupack1_4");
            pBall.score = Content.Load<SoundEffect>("Sounds\\rags\\x");

            menuSelect = Content.Load<SoundEffect>("Sounds\\menupack1_9");
            menuNavigate = Content.Load<SoundEffect>("Sounds\\menupack1_5");
            backgroundMusic = Content.Load<Song>("Sounds\\p1");
            MediaPlayer.Volume = 0.5f;

            //the avatars for the ball and paddles are initialized
            pBall.avatar = Content.Load<Texture2D>("rags");

            Texture2D temp = Content.Load<Texture2D>("linked_list");
            rPlayer.pongPaddle.avatar = temp;
            lPlayer.pongPaddle.avatar = temp;

            ups[0].avatar = Content.Load<Texture2D>("lengthen");
            ups[0].augment.avatar = Content.Load<Texture2D>("long_linked_list");
            ups[0].collect = stud;

            ups[1].avatar = Content.Load<Texture2D>("shorten");
            ups[1].augment.avatar = Content.Load <Texture2D>("shortened_linked_list");
            ups[1].collect = scud;

            ups[2].avatar = Content.Load<Texture2D>("speed");
            ups[2].augment.avatar = Content.Load<Texture2D>("flame");
            ups[2].collect = stud;

            setScreen();

        }

        protected void setScreen()
        {
            int maxY = graphics.GraphicsDevice.Viewport.Height;
            int maxX = graphics.GraphicsDevice.Viewport.Width;

            lPlayer.currentPU.augment.box = new BoundingBox(new Vector3(10f, (maxY / 2f)- (lPlayer.getHeight() / 2f), 0f),
               new Vector3(10f + lPlayer.getWidth(), (maxY / 2f) + (lPlayer.getHeight() / 2f), 0f));
            
            rPlayer.currentPU.augment.box = new BoundingBox(new Vector3(maxX - 10f - rPlayer.getWidth(), (maxY / 2f) - (rPlayer.getHeight() / 2f), 0f),
                new Vector3(maxX - 10f, (maxY / 2f) + (rPlayer.getHeight() / 2f), 0f));

            //only set equal to the augment if a powerup is not in effect
            if (!lPlayer.currentPU.active)
                lPlayer.pongPaddle.box = new BoundingBox(lPlayer.currentPU.augment.box.Min, lPlayer.currentPU.augment.box.Max);
            else
                lPlayer.pongPaddle.box = new BoundingBox(new Vector3(10f, (maxY / 2f) - (lPlayer.pongPaddle.avatar.Height / 2f), 0f), new Vector3(10f + lPlayer.pongPaddle.avatar.Width, (maxY / 2f) + (lPlayer.pongPaddle.avatar.Height / 2f), 0f));

            if(!rPlayer.currentPU.active)
                rPlayer.pongPaddle.box = new BoundingBox(rPlayer.currentPU.augment.box.Min, rPlayer.currentPU.augment.box.Max);
            else
                rPlayer.pongPaddle.box = new BoundingBox(new Vector3(maxX - 10f - rPlayer.pongPaddle.avatar.Width, (maxY /2f) - (rPlayer.pongPaddle.avatar.Height / 2f), 0f), new Vector3(maxX - 10f, (maxY /2f) + (rPlayer.pongPaddle.avatar.Height / 2f), 0f));
            
            //the positions of the ball are set
            pBall.shell = new BoundingSphere(new Vector3(400.0f, 400.0f, 0.0f), (float)pBall.avatar.Height / 2.0f);
            //the starting direction of the ball is set
            pBall.speed.X *= getRandomDirection();
            pBall.speed.Y *= getRandomDirection();
            
            pBall.resetRotation();

        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        private void checkWinner()
        {
            if (lPlayer.score == scoreLimit)
            {
                winner = "Player 1";
                gameOver = true;
            }
            else if (rPlayer.score == scoreLimit)
            {
                winner = "Player 2";
                gameOver = true;
            }

        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            checkWinner();

            scrollY -= 1;
            scrollY2 -= 1;

            if (scrollY2 + backgroundHeight <= 0)
            {
                scrollY2 = backgroundHeight;
            }
            else if (scrollY + backgroundHeight <= 0)
            {
                scrollY = backgroundHeight;
            }


            KeyboardState currentKeyboardState = Keyboard.GetState();

            if (inMainMenu)
            {
                mainMenu.Update(gameTime);

                if (currentKeyboardState.IsKeyDown(Keys.Down) && !previousKeyboardState.IsKeyDown(Keys.Down))
                {
                    if (soundOn)
                    {
                        menuNavigate.Play(0.45f, 0.0f, 0.0f);
                    }
                    mainMenu.selectNext();
                }
                else if (currentKeyboardState.IsKeyDown(Keys.Up) && !previousKeyboardState.IsKeyDown(Keys.Up))
                {
                    if (soundOn)
                    {
                        menuNavigate.Play(0.45f, 0.0f, 0.0f);
                    }
                    mainMenu.selectPrev();
                }

                if(currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                {
                    if (soundOn)
                    {
                        menuSelect.Play();
                    }

                    if (mainMenu.getSelectedName() == "New Game")
                    {
                        inMainMenu = false;
                        inCredits = false;
                        inOptions = false;
                        inControls = false;
                        gameOver = false;
                        scrollY = 0;
                        scrollY2 = backgroundHeight;
                        lPlayer.score = 0;
                        rPlayer.score = 0;

                        rPlayer.currentPU.active = false;
                        lPlayer.currentPU.active = false;

                        millisecondsUntilPowerup = getRandomPowerupTime();
                        millisecondsLastPowerup = 0;
                        timer = 0;

                        if (musicOn)
                        {
                            MediaPlayer.Play(backgroundMusic);
                        }
                        if (soundOn)
                        {
                            see.Play();
                        }
                    }
                    else if (mainMenu.getSelectedName() == "Controls")
                    {
                        inMainMenu = false;
                        inCredits = false;
                        inOptions = false;
                        inControls = true; ;
                    }
                    else if (mainMenu.getSelectedName() == "Options")
                    {
                        inMainMenu = false;
                        inCredits = false;
                        inOptions = true;
                        inControls = false;
                    }
                    else if (mainMenu.getSelectedName() == "Credits")
                    {
                        inMainMenu = false;
                        inCredits = true;
                        inOptions = false;
                        inControls = false;
                    }
                    else if (mainMenu.getSelectedName() == "Exit")
                    {
                        this.Exit();
                    }
                }

                previousKeyboardState = currentKeyboardState;
            }
            else if (inControls || inCredits)
            {
                if (currentKeyboardState.IsKeyDown(Keys.Home) && !previousKeyboardState.IsKeyDown(Keys.Home))
                {
                    if (soundOn)
                    {
                        menuSelect.Play();
                    }

                    inMainMenu = true;
                    inCredits = false;
                    inOptions = false;
                    inControls = false;
                }
            }
            else if (inOptions)
            {
                optionsMenu.Update(gameTime);

                if (currentKeyboardState.IsKeyDown(Keys.Down) && !previousKeyboardState.IsKeyDown(Keys.Down))
                {
                    if (soundOn)
                    {
                        menuNavigate.Play(0.45f, 0.0f, 0.0f);
                    }
                    optionsMenu.selectNext();
                }
                else if (currentKeyboardState.IsKeyDown(Keys.Up) && !previousKeyboardState.IsKeyDown(Keys.Up))
                {
                    if (soundOn)
                    {
                        menuNavigate.Play(0.45f, 0.0f, 0.0f);
                    }
                    optionsMenu.selectPrev();
                }

                if (currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                {
                    if (soundOn)
                    {
                        menuSelect.Play();
                    }

                    if (optionsMenu.getSelectedName() == "Score:")
                    {
                        if (scoreLimit == 3)
                        {
                            scoreLimit = 5;
                        }
                        else if (scoreLimit == 5)
                        {
                            scoreLimit = 7;
                        }
                        else if (scoreLimit == 7)
                        {
                            scoreLimit = 10;
                        }
                        else if (scoreLimit == 10)
                        {
                            scoreLimit = 15;
                        }
                        else if (scoreLimit == 15)
                        {
                            scoreLimit = 3;
                        }
                    }
                    else if (optionsMenu.getSelectedName() == "Sound:")
                    {
                        if (soundOn)
                        {
                            soundOn = false;
                            currentSoundState = "Off";
                        }
                        else
                        {
                            soundOn = true;
                            currentSoundState = "On";
                        }
                    }
                    if (optionsMenu.getSelectedName() == "Music:")
                    {
                        if (musicOn)
                        {
                            musicOn = false;
                            currentMusicState = "Off";
                            MediaPlayer.Stop();
                        }
                        else
                        {
                            musicOn = true;
                            currentMusicState = "On";
                            MediaPlayer.Play(backgroundMusic);
                        }
                    }
                    if (optionsMenu.getSelectedName() == "Resolution:")
                    {
                        if (windowWidth == 800)
                        {
                            windowWidth = 1024;
                            windowHeight = 768;
                        }
                        else if (windowWidth == 1024 && windowHeight == 768)
                        {
                            windowWidth = 1280;
                            windowHeight = 1024;
                        }
                        else if (windowWidth == 1280)
                        {
                            windowWidth = 800;
                            windowHeight = 600;
                        }

                        changeResolution();
                    }
                    if (optionsMenu.getSelectedName() == "Fullscreen:")
                    {
                        if (fullScreen)
                        {
                            fullScreen = false;
                            fullScreenState = "No";
                        }
                        else
                        {
                            fullScreen = true;
                            fullScreenState = "Yes";
                        }

                        changeResolution();
                    }
                    else if (optionsMenu.getSelectedName() == "Back")
                    {
                        inMainMenu = true;
                        inCredits = false;
                        inOptions = false;
                        inControls = false;
                    }
                }

                previousKeyboardState = currentKeyboardState;
            }
            else if (gameOver)
            {
                if (currentKeyboardState.IsKeyDown(Keys.Home) && !previousKeyboardState.IsKeyDown(Keys.Home))
                {
                    inMainMenu = true;
                    inCredits = false;
                    inOptions = false;
                    inControls = false;
                    gameOver = false;
                    
                    if (soundOn)
                    {
                        high.Play();
                    }
                }
            }
            else
            {
                previousTimer = timer;
                timer += (int)gameTime.ElapsedGameTime.TotalMilliseconds;
                //if a powerup has been generated, then another powerup should be scheduled
                //a random powerup should be activated
                if ((float)timer - millisecondsLastPowerup >= millisecondsUntilPowerup)
                {
                    millisecondsLastPowerup = (float)timer;
                    millisecondsUntilPowerup = getRandomPowerupTime();
                    powerIndex = rGen.Next(0, 3);
                    ups[powerIndex].active = true;
                    //the starting coordinates and starting direction are set
                    ups[powerIndex].shell = new BoundingSphere(new Vector3(graphics.GraphicsDevice.Viewport.Width / 2f, graphics.GraphicsDevice.Viewport.Height / 2f, 0f), ups[powerIndex].avatar.Height / 2f);
                    ups[powerIndex].speed.X *= getRandomDirection();
                    ups[powerIndex].speed.Y *= getRandomDirection();

                }

                if (currentKeyboardState.IsKeyDown(Keys.Home) && !previousKeyboardState.IsKeyDown(Keys.Home))
                {
                    inMainMenu = true;
                    MediaPlayer.Stop();
                }

                if (currentKeyboardState.IsKeyDown(Keys.N) && !previousKeyboardState.IsKeyDown(Keys.N))
                {
                    if (soundOn)
                    {
                        soundOn = false;
                        currentSoundState = "Off";
                    }
                    else
                    {
                        soundOn = true;
                        currentSoundState = "On";
                    }
                }

                if (currentKeyboardState.IsKeyDown(Keys.M) && !previousKeyboardState.IsKeyDown(Keys.M))
                {
                    if (musicOn)
                    {
                        musicOn = false;
                        currentMusicState = "Off";
                        MediaPlayer.Stop();
                    }
                    else
                    {
                        musicOn = true;
                        currentMusicState = "On";
                        MediaPlayer.Play(backgroundMusic);
                    }

                }

                UpdateLPaddle(currentKeyboardState);
                UpdateRPaddle(currentKeyboardState);
                //if the ball is invalid (a player has scored) the screen is reset
                if (!pBall.UpdateBall(graphics, rPlayer, lPlayer, (timer - previousTimer) / 1000f, soundOn))
                    setScreen();

                rPlayer.updatePowerUp((float)(timer - previousTimer));
                lPlayer.updatePowerUp((float)(timer - previousTimer));

                previousKeyboardState = currentKeyboardState;

                if (ups[powerIndex].active)
                    ups[powerIndex].UpdatePowerUp(graphics, rPlayer, lPlayer, (timer - previousTimer) / 1000f);

            }

            base.Update(gameTime);

        }

        void UpdateLPaddle(KeyboardState currentKeyboardState)
        {
            float MaxY =
                (float)graphics.GraphicsDevice.Viewport.Height;
            float MinY = 0.0f;

            //left player uses W and S keys to move the paddle
            if (currentKeyboardState.IsKeyDown(Keys.W))
            {
                //move left paddle up
                lPlayer.changePosition(lPlayer.getSpeed() * -1);
                //make sure the paddle hasn't gone off the screen
                if (lPlayer.getUpperBound() < MinY)
                    lPlayer.changePosition(new Vector2(0.0f, MinY - lPlayer.getUpperBound()));

            }

            else if (currentKeyboardState.IsKeyDown(Keys.S))
            {
                // move left paddle down
                lPlayer.changePosition(lPlayer.getSpeed());
                //make sure the paddle hasn't gone off the screen
                if (lPlayer.getLowerBound() > MaxY)
                    lPlayer.changePosition(new Vector2(0.0f, MaxY - lPlayer.getLowerBound()));

            }
        }

        void UpdateRPaddle(KeyboardState currentKeyboardState)
        {
            float MaxY =
                (float)graphics.GraphicsDevice.Viewport.Height;
            float MinY = 0.0f;

            //right player moves his paddle using the up and down arrow keys
            if (currentKeyboardState.IsKeyDown(Keys.Up))
            {
                // move right paddle up
                rPlayer.changePosition(rPlayer.getSpeed() * -1);
                //make sure the paddle hasn't gone off the screen
                if (rPlayer.getUpperBound() < MinY)
                    rPlayer.changePosition(new Vector2(0.0f, MinY - rPlayer.getUpperBound()));

            }

            else if (currentKeyboardState.IsKeyDown(Keys.Down))
            {
                // move right paddle down
                rPlayer.changePosition(rPlayer.getSpeed());
                //make sure the paddle hasn't gone off the screen
                if (rPlayer.getLowerBound() > MaxY)
                    rPlayer.changePosition(new Vector2(0.0f, MaxY - rPlayer.getLowerBound()));

            }

        }

        //returns a random time until the next powerup
        int getRandomPowerupTime()
        {
            return rGen.Next(10000, 20000);

        }

        //returns a random orientation
        int getRandomDirection()
        {
            if (rGen.Next(0, 2) == 0)
                return 1;

            return -1;

        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.DarkCyan);
            int height = GraphicsDevice.Viewport.Height;
            int width = GraphicsDevice.Viewport.Width;

            //spriteBatch.Begin();
            if(inMainMenu)
            {
                mainMenu.Draw(spriteBatch);
                spriteBatch.Begin();
                    spriteBatch.Draw(greece_rags, new Vector2(0, windowHeight - greece_rags.Height), Color.White);
                    spriteBatch.Draw(logo, new Vector2((width - logo.Width) / 2, 25), Color.White);
                    spriteBatch.DrawString(Arial, "UP and DOWN navigate the menu.", new Vector2(windowWidth * 0.25f, 450), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "ENTER selects an option.", new Vector2(windowWidth * 0.25f, 500), Color.GhostWhite);
                spriteBatch.End();
            }
            else if(inCredits)
            {
                spriteBatch.Begin();
                    spriteBatch.Draw(logo, new Vector2((width - logo.Width) / 2, 25), Color.White);
                    spriteBatch.Draw(address_rags, new Vector2(windowWidth - address_rags.Width, windowHeight - address_rags.Height), Color.White);
                    spriteBatch.DrawString(Arial, "Created by Benjamin Hale and Weston Castleberg", new Vector2(50, 100), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "Additional Credits:", new Vector2(50, 200), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "Voice Acting: Scott Ragsdale", new Vector2(50, 225), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "Math Help: Aaron Burns", new Vector2(50, 250), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "Music By: Ben Hale", new Vector2(50, 275), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "Lab Monster: Logan McLain", new Vector2(50, 300), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "Additional Resources:", new Vector2(50, 350), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "http://www.ziggyware.com/readarticle.php?article_id=188", new Vector2(50, 375), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "http://www.3dtextmaker.com/image_editor.html", new Vector2(50, 400), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "http://www.pandora.com/", new Vector2(50, 425), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "Press HOME to return to the main menu.", new Vector2(50, 550), Color.GhostWhite);
                spriteBatch.End();
            }
            else if (inControls)
            {
                spriteBatch.Begin();
                    spriteBatch.Draw(arab_rags, new Vector2(windowWidth - arab_rags.Width, windowHeight - arab_rags.Height), Color.White);
                    spriteBatch.DrawString(Arial, "Player 1 (left):     Up: W       Down: S", new Vector2(50, 50), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "Player 2 (right):     Up: UP       Down: DOWN", new Vector2(50, 100), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "During gameplay, M toggles background music", new Vector2(50, 200), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "     and N toggles sound effects.", new Vector2(50, 250), Color.GhostWhite);
                    spriteBatch.DrawString(Arial, "Press HOME to return to the main menu.", new Vector2(50, 350), Color.GhostWhite); 
                spriteBatch.End();
            }
            else if (inOptions)
            {
                optionsMenu.Draw(spriteBatch);
                spriteBatch.Begin();
                    spriteBatch.Draw(toga_rags, new Vector2(windowWidth - toga_rags.Width, windowHeight - toga_rags.Height), Color.White);
                    spriteBatch.DrawString(Arial, scoreLimit.ToString(), new Vector2(300, 50), Color.LightGreen);
                    spriteBatch.DrawString(Arial, currentSoundState, new Vector2(300, 100), Color.LightGreen);
                    spriteBatch.DrawString(Arial, currentMusicState, new Vector2(300, 150), Color.LightGreen);
                    spriteBatch.DrawString(Arial, (windowWidth.ToString() + " x " + windowHeight.ToString()), new Vector2(300,200), Color.LightGreen);
                    spriteBatch.DrawString(Arial, fullScreenState, new Vector2(300, 250), Color.LightGreen);
                spriteBatch.End();
            }
            else if (gameOver)
            {
                GraphicsDevice.Clear(Color.DarkCyan);
                
                spriteBatch.Begin();
                    spriteBatch.DrawString(Arial, (winner + " wins!"), new Vector2(width/4, height/2), Color.White);
                    spriteBatch.DrawString(Arial, "Press HOME to return to the menu.", new Vector2(width/2, height / 2), Color.White);
                spriteBatch.End();
            }
            else
            {
                spriteBatch.Begin();
                    spriteBatch.Draw(scrollingBackground, new Vector2((width / 2f) - (scrollingBackground.Width / 2f), scrollY), Color.White);
                    spriteBatch.Draw(scrollingBackground2, new Vector2((width / 2f) - (scrollingBackground.Width / 2f), scrollY2), Color.White);
                    pBall.draw(spriteBatch);
                    rPlayer.draw(spriteBatch);
                    lPlayer.draw(spriteBatch);
                    spriteBatch.DrawString(Impact, lPlayer.score.ToString(), new Vector2(width / 8f, 10.0f), Color.White);
                    spriteBatch.DrawString(Impact, rPlayer.score.ToString(), new Vector2(7 * (width / 8f), 10.0f), Color.White);
                    ups[powerIndex].draw(spriteBatch);
                spriteBatch.End();
            }

            base.Draw(gameTime);
        }
    }
}