﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Design;

namespace RagPong
{
    class ball : Bounceable
    {
        public float rotationAngle;
        public float rotationDirection;
        public SoundEffect paddleHit;
        public SoundEffect score;

        //default constructor: default speed assigned, shell initialized, avatar left NULL
        public ball()
        {
            setDefaultSpeed();
            shell = new BoundingSphere();
            resetRotation();
        }

        //second constructor: speed assigned by parameter, shell initialized, avatar left NULL   
        public ball(Vector2 s)
        {
            speed = s;
            shell = new BoundingSphere();
            resetRotation();

        }

        //third constructor: speed, shell assigned.  avatar left NULL
        public ball(Vector2 s, Vector2 center, float radius)
        {
            speed = s;
            //the center and radius passed in are used to initialize the bounding sphere.
            //since this object is 2d, the z field is a 0
            shell = new BoundingSphere(new Vector3(center.X, center.Y, 0), radius);
            resetRotation();

        }

        //fourth constructor: speed, shell, avatar all assigned
        public ball(Vector2 s, Vector2 center, float radius, Texture2D a)
        {
            //the center and radius passed in are used to initialize the bounding sphere.
            //since this object is 2d, the z field is a 0
            shell = new BoundingSphere(new Vector3(center.X, center.Y, 0), radius);
            avatar = a;
            resetRotation();

        }

        //this function draws the sprite to the screen
        public override void draw(SpriteBatch sB)
        {
            // no rotation, good collision
            //sB.Draw(avatar, new Vector2(shell.Center.X - shell.Radius, shell.Center.Y - shell.Radius), Color.White);

            // final?
            sB.Draw(avatar,
                new Vector2(shell.Center.X, shell.Center.Y), null, Color.White, rotationAngle,
                new Vector2((float)(avatar.Width / 2), (float)(avatar.Height / 2)),
                new Vector2(1f, 1f), SpriteEffects.None, 0f);

        }

        //this function updates the position of the ball on the screen and determines
        //whether the ball has changed directions or collided with a paddle. whether
        //or not the ball is still valid is returned.  if a player has scored, the 
        //ball is no longer valid
        public bool UpdateBall(GraphicsDeviceManager graphics, player rPlayer, player lPlayer, double elapsed, bool soundOn)
        {
            // Move the sprite by speed, scaled by elapsed time.
            shell.Center = new Vector3(shell.Center.X + (speed.X * (float)elapsed),
                shell.Center.Y + (speed.Y * (float)elapsed), 0);

            float circle = MathHelper.Pi * 2;
            rotationAngle += ( (float)(elapsed % circle) * rotationDirection);// rotation;

            float MaxX =
                graphics.GraphicsDevice.Viewport.Width;
            float MinX = 0;
            float MaxY =
                graphics.GraphicsDevice.Viewport.Height;
            float MinY = 0;

            bool isValid = true;

            if (Collision(rPlayer.getPaddle()))
            {
                //the angle of bounce is determined
                double theta = (double)getBounceAngle(lPlayer);
                double velocity = Math.Sqrt(Math.Pow(speed.X, 2) + Math.Pow(speed.Y, 2));
                if (speed.Y < 0)
                    speed.Y = (float)(velocity * Math.Sin(theta)) * -1;
                else
                    speed.Y = (float)(velocity * Math.Sin(theta));
                speed.X = (float)(velocity * Math.Cos(theta));
                //X speed must be negative, because it is going away from the right paddle
                //now.  the angle of bounce may produce a negative X
                if (speed.X > 0)
                    speed.X *= -1;

                //speed.Y *= (getBounceAngle(rPlayer) * -1);
                //the speed will double after 5 collisions
                speed.X *= 1.149f;
                speed.Y *= 1.149f;
                //the ball should bounce clear of the paddle
                changePosition(new Vector2((rPlayer.getLeftBound() - getRightBound()) - 1, 0.0f));
                //the ball reverses its rotation
                rotationDirection *= -1;

                if (soundOn)
                {
                    paddleHit.Play();
                }
            }

            else if (Collision(lPlayer.getPaddle()))
            {
                //the angle of bounce is determined
                double theta = (double)getBounceAngle(lPlayer);
                double velocity = Math.Sqrt(Math.Pow(speed.X, 2) + Math.Pow(speed.Y, 2));
                if (speed.Y < 0)
                    speed.Y = (float)(velocity * Math.Sin(theta)) * -1;
                else
                    speed.Y = (float)(velocity * Math.Sin(theta));
                speed.X = (float)(velocity * Math.Cos(theta));
                //X speed must be positive because the ball will be heading away from the 
                //left paddle now
                if (speed.X < 0)
                    speed.X *= -1;

                //speed.Y *= (getBounceAngle(lPlayer));
                //the speed will double after 5 collisions
                speed.X *= 1.149f;
                speed.Y *= 1.149f;
                //the ball should bounce clear of the paddle
                changePosition(new Vector2((lPlayer.getRightBound() - getLeftBound()) + 1, 0.0f));
                //the ball reverses its rotation
                rotationDirection *= -1;

                if (soundOn)
                {
                    paddleHit.Play();
                }
            }

            else if (getRightBound() > MaxX)
            {
                //score is incremented, and the course is reset
                lPlayer.score += 1;
                isValid = false;
                setDefaultSpeed();

                if (soundOn)
                {
                    score.Play();
                }
            }

            else if (getLeftBound() < MinX)
            {
                //score is incrmented, and the course is reset
                rPlayer.score += 1;
                isValid = false;
                setDefaultSpeed();

                if (soundOn)
                {
                    score.Play();
                }
            }

            else if (getLowerBound() > MaxY)
            {
                speed.Y *= -1;
                changePosition(new Vector2(MaxY - getLowerBound(), 0.0f));

                if (soundOn)
                {
                    paddleHit.Play();
                }
            }

            else if (getUpperBound() < MinY)
            {
                speed.Y *= -1;
                changePosition(new Vector2(MinY - getUpperBound(), 0.0f));

                if (soundOn)
                {
                    paddleHit.Play();
                }
            }

            checkMinimumBallSpeed();

            return isValid;

        }

        //this function returns the angle at which a ball should bounce off the paddle,
        //depending upon where in the paddle the ball hit
        protected float getBounceAngle(player p)
        {
            //point of contact
            float pointYC;
            //center of paddle
            float pointYCenter = (p.getLowerBound() - p.getUpperBound()) / 2.0f;
            //the closest edge of the paddle
            float pointYEdge;
            //the maximum angle of bounce;
            float maxAngle = 30.0f;
            //it is assumed that a collision is true            

            if (shell.Center.Y < p.getUpperBound())
            {
                pointYC = p.getUpperBound();
                pointYEdge = p.getUpperBound();

            }

            else if (shell.Center.Y > p.getLowerBound())
            {
                pointYC = p.getLowerBound();
                pointYEdge = p.getLowerBound();

            }

            else if (p.getLowerBound() - shell.Center.Y > shell.Center.Y - p.getUpperBound())
            {
                pointYC = shell.Center.Y;
                pointYEdge = p.getUpperBound();

            }

            else
            {
                pointYC = shell.Center.Y;
                pointYEdge = p.getLowerBound();
            }

            //the equation for the angle of bounce.  the farther toward the edges the
            //ball collides with the paddle, the more acute the angle of bounce will be
            return (90.0f - (((pointYEdge - pointYC) / (pointYEdge - pointYCenter)) *
                (90.0f - maxAngle)));

        }

       /* //checks the increase in ball speed. if the increase is less than 1.5, then the
        //newS is returned, otherwise the maximum increase in speed is returned
        private Vector2 checkMaximumBallSpeed(Vector2 oldS, Vector2 newS)
        {

        }*/

        protected void checkMinimumBallSpeed()
        {
            //in order to keep the ball from becoming too vertically moving,
            //the ball will curve when it's not going fast enough on the y-axis

            //whether the ball is currently going left or right determines whether
            //adding or subtracting from the x-value of speed will increase the 
            //velocity
            if (Math.Abs(speed.X) < Math.Abs(speed.Y) / 2f)
            {
                if (speed.X < 0)
                    speed.X -= 10;
                else
                    speed.X += 10;

                if (speed.Y < 0)
                    speed.Y += 10;
                else
                    speed.Y -= 10;

            }

            else if (Math.Abs(speed.Y) < Math.Abs(speed.X) / 2f)
            {
                if (speed.Y < 0)
                    speed.Y -= 10;
                else
                    speed.Y += 10;

                if (speed.X < 0)
                    speed.X += 10;
                else
                    speed.X -= 10;

            }

        }

        //sets the speed to default
        public override void setDefaultSpeed()
        {
            speed = new Vector2(200f, 200f);
        }

        public void resetRotation()
        {
            rotationAngle = 0f;
            rotationDirection = 1f;

        }
    }
}