﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Design;

namespace RagPong
{
    class paddle
    {
        public Vector2 speed;
        public Texture2D avatar;
        public BoundingBox box;

        //default constructor: speed given default value, box initialized, avatar left NULL
        public paddle()
        {
            setDefaultSpeed();
            box = new BoundingBox();

        }

        //second constructor: speed assigned by parameter, box initialized, avatar
        //left NULL
        public paddle(float s)
        {
            //a paddle only moves up and down, and therefore has an x-axis speed of 0, only
            //the y-axis speed must be set
            speed = new Vector2(0.0f, s);
            box = new BoundingBox();

        }

        //third constructor: speed, box assigned.  avatar left NULL
        public paddle(float s, Vector2 box1, Vector2 box2)
        {
            //a paddle only moves up and down, and therefore has an x-axis speed of 0, only
            //the y-axis speed must be set
            speed = new Vector2(0.0f, s);
            //the vectors passed in represent the edges of a 2d box.  the BoundingBox class 
            //represents a 3d cube, however, so 0s are filled in for the x values
            box = new BoundingBox(new Vector3(box1.X, box1.Y, 0.0f), new Vector3(box2.X, box2.Y, 0.0f));

        }

        //fourth constructor: speed, box, and avatar all assigned
        public paddle(float s, Vector2 box1, Vector2 box2, Texture2D a)
        {
            //a paddle only moves up and down, and therefore has an x-axis speed of 0, only
            //the y-axis speed must be set
            speed = new Vector2(0.0f, s);
            //the vectors passed in represent the edges of a 2d box.  the BoundingBox class 
            //represents a 3d cube, however, so 0s are filled in for the x values
            box = new BoundingBox(new Vector3(box1.X, box1.Y, 0.0f), new Vector3(box2.X, box2.Y, 0.0f));
            avatar = a;

        }

        //returns the top x-coordinate for the box member
        public float getUpperBound()
        {
            return box.Min.Y;
        }

        //returns the bottom x-coordinate for the box member
        public float getLowerBound()
        {
            return box.Max.Y;
        }

        //returns the left y-coordinate of the box member
        public float getLeftBound()
        {
            return box.Min.X;
        }

        //returns the right y-coordinate of the box member
        public float getRightBound()
        {
            return box.Max.X;
        }

        //changes the x position of the box by the amount passed in
        public void changeX(float x)
        {
            box.Min.X += x;
            box.Max.X += x;

        }

        //changes the y position of the box by the amount passed in
        public void changeY(float y)
        {
            box.Min.Y += y;
            box.Max.Y += y;

        }

        //changes the position of the box by the amounts directed by
        //the vectors passed in
        public void changePosition(Vector2 change)
        {
            changeX(change.X);
            changeY(change.Y);

        }

        //this function copies the data members of the object to another
        //paddle object passed in
        public void copyPaddle(ref paddle r)
        {
            r.avatar = avatar;
            r.box = box;
            r.speed = speed;

        }

        public void setDefaultSpeed()
        {
            speed = new Vector2(0f, 6f);
        }

    }

}