﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Design;

namespace RagPong
{
    class player
    {
        public int score;
        public paddle pongPaddle;
        public powerUp currentPU;
        private bool flicker;
        private int flickerCount;

        //default constructor: initializes paddle, sets score to 0
        public player()
        {
            score = 0;
            pongPaddle = new paddle();
            currentPU = new powerUp();
            currentPU.active = false;

        }

        //get upper bound of player's position.  if a powerup is in 
        //effect, the upper bound of the powerUp is used
        public float getUpperBound()
        {
            if (currentPU.active)
                return currentPU.augment.getUpperBound();

            return pongPaddle.getUpperBound();

        }

        //get left bound of the player's position. if a powerup is in
        //effect, the left bound of a powerup is used
        public float getLeftBound()
        {
            if (currentPU.active)
                return currentPU.augment.getLeftBound();

            return pongPaddle.getLeftBound();

        }

        //get right bound of the player's position. if a powerup is in
        //effect, the right bound of a powerup is used
        public float getRightBound()
        {
            if (currentPU.active)
                return currentPU.augment.getRightBound();

            return pongPaddle.getRightBound();

        }

        //get lower bound of the player's position. if a powerup is in
        //effect, the lower bound of a powerup is used
        public float getLowerBound()
        {
            if (currentPU.active)
                return currentPU.augment.getLowerBound();

            return pongPaddle.getLowerBound();

        }

        //applies the new powerup
        public void applyNewPowerUp(powerUp pUp)
        {
            //claims the powerup so it no longer bounces around
            pUp.claimed = true;

            currentPU.active = true;
            currentPU.augment.avatar = pUp.augment.avatar;
            currentPU.augment.speed = pUp.augment.speed;
            currentPU.setDefaultExpirationTime();
            

            //flicker is set to false
            flicker = false;
            flickerCount = 0;

            //avatar and bounding box are modified
            if (currentPU.augment.avatar.Height >= pongPaddle.avatar.Height)
            {
                float minX = pongPaddle.box.Min.X;
                float minY = pongPaddle.box.Min.Y - ((currentPU.augment.avatar.Height - pongPaddle.avatar.Height) / 2f);
                float maxX = pongPaddle.box.Max.X;
                float maxY = pongPaddle.box.Max.Y + ((currentPU.augment.avatar.Height - pongPaddle.avatar.Height) / 2f);

                currentPU.augment.box = new BoundingBox(new Vector3(minX, minY, 0f), new Vector3(maxX, maxY, 0f));

            }

            else
            {
                float minX = pongPaddle.box.Min.X;
                float minY = pongPaddle.box.Min.Y + ((pongPaddle.avatar.Height - currentPU.augment.avatar.Height) / 2f);
                float maxX = pongPaddle.box.Max.X;
                float maxY = pongPaddle.box.Max.Y - ((pongPaddle.avatar.Height - currentPU.augment.avatar.Height) / 2f);

                currentPU.augment.box = new BoundingBox(new Vector3(minX, minY, 0f), new Vector3(maxX, maxY, 0f));

            }

        }

        //update the time left on the powerup
        public void updatePowerUp(float elapsed)
        {
            if (currentPU.active)
            {
                currentPU.expirationTime -= elapsed;
                //if powerup is expired, turn if off
                if (currentPU.expirationTime <= 0)
                    unApplyPowerUp();

            }

        }

        //deactive the powerup and remove its effects
        public void unApplyPowerUp()
        {
            currentPU.active = false;
            flicker = false;
            flickerCount = 0;
            currentPU.augment.box = new BoundingBox(pongPaddle.box.Min, pongPaddle.box.Max);

        }

        //change the position of the paddle by the amount
        //passed in
        public void changePosition(Vector2 change)
        {
            //if a powerup is active, the powerup's position
            //must be changed as well
            currentPU.augment.changePosition(change);

            pongPaddle.changePosition(change);

        }

        //gets the height of the avatar being used
        public float getHeight()
        {
            if (currentPU.active)
                return currentPU.augment.avatar.Height;

            return pongPaddle.avatar.Height;

        }

        //gets the width of the avatar being used
        public float getWidth()
        {
            if (currentPU.active)
                return currentPU.augment.avatar.Width;

            return pongPaddle.avatar.Width;

        }

        //draws the player's paddle
        public void draw(SpriteBatch sB)
        {
            if (!flicker && currentPU.active)
            {
                sB.Draw(currentPU.augment.avatar, new Vector2(currentPU.augment.box.Min.X, currentPU.augment.box.Min.Y), Color.White);

                if (currentPU.expirationTime <= 2000f)
                {
                    flickerCount++;
                    if (flickerCount >= 50)
                        flicker = true;

                }

            }

            else
            {
                sB.Draw(pongPaddle.avatar, new Vector2(pongPaddle.box.Min.X, pongPaddle.box.Min.Y), Color.White);

                if (currentPU.active)
                {
                    flickerCount++;
                    if (flickerCount >= 50)
                        flicker = false;

                }

            }

        }

        //returns the paddle being used by the class
        public paddle getPaddle()
        {
            if (currentPU.active)
                return currentPU.augment;

            return pongPaddle;

        }

        //returns the speed of the paddle in use
        public Vector2 getSpeed()
        {
            if (currentPU.active)
                return currentPU.augment.speed;

            return pongPaddle.speed;

        }

    }

}